#!/usr/bin/perl
#
# Sensor for gathering PosgreSQL performance data for Zabbix
# Copyright (c) 2009 Hrvoje Sute
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Changes and Modifications
# * Fri May  1 21:06:11 CEST 2009
#       - rewritten from sh to perl
#

use strict;
use warnings;

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $send_file="/var/tmp/zabbixSenderFileMySQL";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd -i $send_file";
my $inputString = ''; 
my $hostname = `/bin/hostname -f`; 
chomp($hostname);

# signal handling
$SIG{'ALRM'} = sub {
        local $SIG{TERM} = 'IGNORE';
        kill TERM => -$$;
        print "0\n";
};

local $SIG{TERM} = sub {
        local $SIG{TERM} = 'IGNORE';
        kill TERM => -$$;
        print "0\n";
};

alarm 25;

# function to add zeros
sub append0 {
  my $tmp=shift;
  $tmp =~ s/\.//;
  $tmp =~ s/K/0/i;
  $tmp =~ s/M/0000/i;
  $tmp =~ s/G/0000000/i;
  $tmp =~ s/T/0000000000/i;
  $tmp;
}

# MySQL access data
my $mysqlreport='/usr/lib/zabbix-agent/mysqlreport';
my $mysqluser='zabbix-agentd';
my $mysqlpass='p4ssw0rdZabbi><';

# generate command
my $report_cmd = "$mysqlreport --user $mysqluser --password \"$mysqlpass\" --no-mycnf --flush-status  2>&1";
my $output = qx($report_cmd);          

# get data
my %data; 
$data{'mysql.version'} = $1 if ($output =~ /MySQL\s+((\d|\.)+)/); 
$data{'mysql.uptime'} = $1.":".$2 if ($output =~ /uptime\s+(\d+)\s(\d+:\d+:\d+)/);

my $outPart = $1 if ($output =~ /(Key __(.|\s)+)_ Questions _/); 
$data{'mysql.keysusedmaxpercent'} = $1 if ($outPart =~ /Buffer used[\w|\.|\s]+\%Used:\s+((\d|\.)+)/);
$data{'mysql.keysusedcurrentpercent'} = $1 if ($outPart =~ /Current[\w|\.|\s]+\%Usage:\s+((\d|\.)+)/);
$data{'mysql.keyswritehit'} = $1 if ($outPart =~ /Write hit\s+((\d|\.)+)\%/);
$data{'mysql.keysreadhit'} = $1 if ($outPart =~ /Read hit\s+((\d|\.)+)\%/);

$outPart = $1 if ($output =~ /(Questions __(.|\s)+)_ SELECT and Sort _/);  
$data{'mysql.questionstotal'} = append0($1) and 
$data{'mysql.questionstotalpersec'} = $3 if ($outPart =~ /Total\s+((\d|\.)+\S{1,2})\s+((\d|\.)+)/);
$data{'mysql.questionsDMS'} = $2 if ($outPart =~ /DMS[\w| |\.]+\/\w +(\%Total:)? +((\d|\.)+)/);
$data{'mysql.questionsSlow'} = $3 and
$data{'mysql.questionsSlowDMS'} = $5 if ($outPart =~ /Slow 10 s[\w|\s]+\s((\d|\.)+\/\w)\s+((\d|\.)+)\s+\%DMS:\s+((\d|\.)+)/);
$data{'mysql.questionsCOM'} = $2 if ($outPart =~ /Com_[\w|\.| |\/]+(\%Total:)? * ((\d|\.)+)/);
$data{'mysql.questionsQCHits'} = $1 if ($outPart =~ /QC Hits[\w|\.| ]+\/\w\s+((\d|\.)+)/);
$data{'mysql.questionsCOM_QUIT'} = $1 if ($outPart =~ /COM_QUIT[\w|\.|\s]+\/\w\s+((\d|\.)+)/);
$data{'mysql.questionsUnknown'} = $1 if ($outPart =~ /Unknown[\w|\.|\s]+\/\w\s+((\d|\.)+)/);
	#								print $output;
$outPart = $1 if ($output =~ /(__ Query Cache __([\w|\.|\/| |\%|:]+\s)+)/); #print $outPart;
$data{'mysql.QCmemmax'} = append0($1)  and
$data{'mysql.QCmemused'} = append0($3) and
$data{'mysql.QCmemusedpercent'} = $5 if ($outPart =~ /Memory usage\s+((\d|\.)+[k|M|G|T])\s+of\s+((\d|\.)+[k|M|G|T])\s+\%Used:\s+((\d|\.)+)/i);
$data{'mysql.QCblockfragment'} = $1 if ($output =~ /Block Fragmnt\s+((\d|\.)+)\%/);
$data{'mysql.QCHits'} = $1 if ($output =~ /Hits\s+(\d+)/);
$data{'mysql.QCInserts'} = $1 if ($output =~ /Inserts\s+(\d+)/);
$data{'mysql.QCInsertPruneRatio'} = $1 if ($output =~ /Insrt:Prune\s+((\d|\.)+:(\d|\.)+)/);
$data{'mysql.QCHitInsertRatio'} = $1 if ($output =~ /Hit:Insert\s+((\d|\.)+:(\d|\.)+)/);

$outPart = $1 if ($output =~ /(Tables __(.|\s)+)_ Connections _/); 
$data{'mysql.tablestotal'} = $2 and
$data{'mysql.tablesopen'} = $1 and 
$data{'mysql.tablesopenpercent'} = $3 if ($outPart =~ /Open\s+(\d+)\s+of\s+(\d+)\s+\%Cache:\s+((\d|\.)+)/);
									
$outPart = $1 if ($output =~ /(Table Locks __(.|\s)+)_ Tables _/); 
$data{'mysql.tablelockswaited'} = $1  if ($outPart =~ /Waited\s+(\d+)/);
$data{'mysql.tablelockswaitedpercent'} = $2 if ($outPart =~ /(Waited[\w|\.|\s|\/]+\%Total:\s+((\d|\.)+))/);
$data{'mysql.tablelocksimmediate'} = $1  if ($outPart =~ /Immediate\s+(\d+)/);

$outPart = $1 if ($output =~ /(Connections __(.|\s)+)_ Created Temp _/); 
$data{'mysql.connectionsusedmax'} = $1 and 
$data{'mysql.connectionsmax'} =$2 and
$data{'mysql.connectionsusedmaxpercent'} =$3 if ($outPart =~ /Max used\s+(\d+)\s+of\s+(\d+)\s+\%Max:\s+((\d|\.)+)/);
$data{'mysql.connectionstotalpersec'} = $1 if ($outPart =~ /Total\s+[\w|\.]+\s+((\d|\.)+)\//);

$outPart = $1 if ($output =~ /(Created Temp __(.|\s)+)_ Threads _/); 
$data{'mysql.tempdisktable'} = $1 if ($outPart =~ /Disk table\s+(\d+)/);
$data{'mysql.temptable'} = append0($1) if ($outPart =~ /Table\s+((\d|\.)+[k|K|M|G|T])/);
$data{'mysql.tempfile'} = $1 if ($outPart =~ /File\s+(\d+)/);

$outPart = $1 if ($output =~ /(Threads __(.|\s)+)_ Aborted _/); 
$data{'mysql.threadsrunning'} = $1 and 
$data{'mysql.threadstotal'} = $2 if ($outPart =~ /Running\s+(\d+)\s+of\s+(\d+)/);
$data{'mysql.threadscached'} = $1 if ($outPart =~ /Cached\s+(\d+)/);
$data{'mysql.threadscachehit'} = $1 if ($outPart =~ /\%Hit:\s+(\d+)/);
 
$outPart = $1 if ($output =~ /(__ Bytes __([\w|\.|\/| ]+\s)+)/); 
$data{'mysql.bytessent'} = append0($1) if ($outPart =~ /Sent\s+((\d|\.)+[k|K|M|G|T])/);
$data{'mysql.bytesreceived'} = append0($1) if ($outPart =~ /Received\s+((\d|\.)+[k|K|M|G|T])/);

# write data to variable
foreach my $key (keys %data){
  $inputString .= $hostname ." ". $key ." ". $data{$key} ."\n" if($data{$key}); #$server ." ". $hostname ." ". $port 
}

# write everything to file
open FH, ">", $send_file or die("Can not open file $send_file!");
print FH $inputString;
close(FH);

# finally, send the data
if ( qx($zabbix_send_command) =~ /Failed(:)? 0/i ) {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("1\n");
   exit(0);
} else {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("0\n");
   exit(-1);
}
#EOF
